import base64
import random
import string
from datetime import datetime

import pyotp
from django.conf import settings
from django.contrib.auth.models import Group
from django.contrib.auth.tokens import default_token_generator
from django.core.mail import send_mail
from django.db.models import Q
from django.utils import translation, timezone
from django.utils.timezone import now
from django.utils.translation import gettext as _
from djoser.compat import get_user_email_field_name, get_user_email
from djoser.conf import settings as djoser_settings
from djoser.utils import ActionViewMixin, logout_user, encode_uid
from rest_framework import generics, status
from rest_framework import views
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from rest_framework_simplejwt.views import TokenViewBase

from moses.conf import settings as moses_settings
from moses.decorators import otp_required
from moses.models import CustomUser
from moses.serializers import PasswordResetSerializer, ShortCustomUserSerializer, \
    TokenObtainPairSerializer, PublicCustomUserSerializer, PinSerializer, MFASerializer

LANGUAGES_LIST = [l[0] for l in moses_settings.LANGUAGE_CHOICES]


class ConfirmPhoneNumber(generics.GenericAPIView):
    permission_classes = (IsAuthenticated,)
    serializer_class = PinSerializer

    def post(self, request):
        candidate_phone_number = request.user.phone_number_candidate
        confirmation_result = request.user.try_to_confirm_phone_number(
            request.data['pin'],
            request.data.get('candidate_pin', '')
        )
        if candidate_phone_number:
            if confirmation_result:
                with translation.override(request.user.preferred_language):
                    send_mail(_("Phone number changed"),
                              _(
                                  f"Your phone number has been changed. If it happened without your desire - contact us by email support@{moses_settings.DOMAIN}."),
                              'noreply@' + moses_settings.DOMAIN, [request.user.email])
                return Response({'result': 'ok'})
        return Response({'non_field_errors': ['invalid_pin']}, status.HTTP_400_BAD_REQUEST)


class ConfirmEmail(generics.GenericAPIView):
    permission_classes = (IsAuthenticated,)
    serializer_class = PinSerializer

    def post(self, request):
        candidate_email = request.user.email_candidate
        confirmation_result = request.user.try_to_confirm_email(request.data['pin'],
                                                                request.data.get('candidate_pin', ''))
        if confirmation_result:
            if candidate_email:
                with translation.override(request.user.preferred_language):
                    send_mail(_("Email changed"),
                              _(
                                  f"Your email has been changed. If it happened without your desire - contact us by email support@{moses_settings.DOMAIN}."),
                              'noreply@' + moses_settings.DOMAIN, [candidate_email])
            return Response({'result': 'ok'})
        return Response({'non_field_errors': ['invalid_pin']}, status.HTTP_400_BAD_REQUEST)


class TokenObtainPairView(TokenViewBase):
    serializer_class = TokenObtainPairSerializer

    def get_serializer_context(self):
        return {'request': self.request}


class ValidateAuthView(generics.GenericAPIView):
    def post(self, request):
        if request.user.is_authenticated:
            if request.user.verify_mfa_otp(request.data.get('mfa_code')):
                return Response({'user_id': request.user.id, 'verified_at': int(datetime.timestamp(now()))})
            else:
                return Response({'non_field_errors': ['invalid_otp']}, status.HTTP_400_BAD_REQUEST)
        return Response({'non_field_errors': ['not_authenticated']}, status.HTTP_400_BAD_REQUEST)


class CheckEmailAvailability(generics.GenericAPIView):
    serializer_class = PublicCustomUserSerializer

    def get(self, request):
        return Response({'result': not CustomUser.objects.filter(email=request.GET.get('email')).exists()},
                        status=status.HTTP_200_OK)


class CheckIsMFAEnabled(generics.GenericAPIView):
    serializer_class = PublicCustomUserSerializer
    permission_classes = list()

    def get(self, request):
        u_qs = CustomUser.objects.filter(phone_number=self.request.GET.get('phone_number'))
        result = u_qs.exists() and bool(u_qs.first().mfa_secret_key)
        return Response({'result': result}, status=status.HTTP_200_OK)


class MFAView(generics.GenericAPIView):
    serializer_class = MFASerializer

    @otp_required
    def post(self, request):
        if request.data['action'] == 'get_key':
            mfa_secret_key = base64.b32encode(
                bytes(''.join(random.choice(string.ascii_letters) for _ in range(100)).encode('utf-8'))).decode('utf-8')
            return Response({'mfa_url': pyotp.totp.TOTP(mfa_secret_key.encode('utf-8'))
                            .provisioning_uri(f"{request.user.first_name} {request.user.last_name}", moses_settings.DOMAIN),
                             'mfa_secret_key': mfa_secret_key})

        elif not request.user.mfa_secret_key and request.data['action'] == 'enable':
            mfa_secret_key = request.data.get('mfa_secret_key')
            otp = request.headers.get('otp', '')
            if pyotp.totp.TOTP(mfa_secret_key.encode('utf-8')).verify(otp):
                request.user.mfa_secret_key = mfa_secret_key
                request.user.save()
                return Response({'success': 'mfa has been successfully disabled'})
            return Response({'non_field_errors': ['invalid_otp']}, status=status.HTTP_400_BAD_REQUEST)

        elif request.user.mfa_secret_key and request.data['action'] == 'disable':
            request.user.mfa_secret_key = ''
            request.user.save()
            return Response({'success': 'mfa has been successfully disabled'})


class ResetPassword(ActionViewMixin, generics.GenericAPIView):
    serializer_class = PasswordResetSerializer
    _users = None

    permission_classes = list()

    def _action(self, serializer):
        if 'email' in serializer.data:
            for user in self.get_users_by_email(serializer.data['email']):
                if user.is_email_confirmed:
                    try:
                        self.send_password_reset_email(user)
                    except ValueError as e:
                        return Response({'error': str(e)},
                                        status=status.HTTP_400_BAD_REQUEST)
                else:
                    return Response({'non_field_errors': ['email_not_activated']},
                                    status=status.HTTP_400_BAD_REQUEST)

        if 'phone_number' in serializer.data:
            users = self.get_users_by_phone_number(serializer.data['phone_number'])
            if not users:
                return Response({'non_field_errors': ['phone_number_not_found']},
                                status=status.HTTP_400_BAD_REQUEST)
            for user in users:
                if user.is_phone_number_confirmed:
                    if not user.last_password_reset_sms_sent_at or (
                            timezone.now() - user.last_password_reset_sms_sent_at).days > 0:
                        try:
                            self.send_password_reset_sms(user)
                        except ValueError as e:
                            return Response({'error': str(e)}, status=status.HTTP_400_BAD_REQUEST)
                    else:
                        return Response({'non_field_errors': ['too_frequent_sms_request']},
                                        status=status.HTTP_400_BAD_REQUEST)
                else:
                    return Response({'non_field_errors': ['phone_number_not_confirmed']},
                                    status=status.HTTP_400_BAD_REQUEST)
        return Response({}, status=status.HTTP_204_NO_CONTENT)

    def get_users_by_email(self, email):
        if self._users is None:
            email_field_name = get_user_email_field_name(CustomUser)
            users = CustomUser._default_manager.filter(**{
                email_field_name + '__iexact': email
            })
            self._users = users
        return self._users

    def get_users_by_phone_number(self, phone_number):
        if self._users is None:
            users = CustomUser._default_manager.filter(**{
                'phone_number__iexact': phone_number
            })
            self._users = users
        return self._users

    def send_password_reset_email(self, user):
        context = {'user': user}
        to = [get_user_email(user)]
        if user.is_email_confirmed:
            with translation.override(user.preferred_language):
                djoser_settings.EMAIL.password_reset(self.request, context).send(to)
        else:
            raise ValueError(_("Email is not confirmed"))

    def send_password_reset_sms(self, user):
        url = settings.URL_PREFIX + '/' + djoser_settings.PASSWORD_RESET_CONFIRM_URL.format(
            token=default_token_generator.make_token(user),
            uid=encode_uid(user.pk))
        moses_settings.SEND_SMS_HANDLER(to=user.phone_number, body=url)
        user.last_password_reset_sms_sent_at = timezone.now()
        user.save()


class SetPasswordView(ActionViewMixin, generics.GenericAPIView):
    """
    Use this endpoint to change user password.
    """
    permission_classes = djoser_settings.PERMISSIONS.set_password

    def get_serializer_class(self):
        if djoser_settings.SET_PASSWORD_RETYPE:
            return djoser_settings.SERIALIZERS.set_password_retype
        return djoser_settings.SERIALIZERS.set_password

    def _action(self, serializer):
        self.request.user.set_password(serializer.data['new_password'])
        self.request.user.save()
        with translation.override(self.request.user.preferred_language):
            send_mail(_("Password changed"),
                      _(
                          "Your password has been changed. "
                          f"If it happened without your desire - contact us by email support@{moses_settings.domain}."),
                      'noreply@' + moses_settings.DOMAIN, [self.request.user.email])
        if djoser_settings.LOGOUT_ON_PASSWORD_CHANGE:
            logout_user(self.request)

        return Response(status=status.HTTP_204_NO_CONTENT)


class RequestPhoneNumberConfirmPin(generics.GenericAPIView):
    serializer_class = ShortCustomUserSerializer

    def post(self, request):
        if (now() - request.user.last_phone_number_confirm_pin_sent).days > 0 and \
                (now() - request.user.last_phone_number_candidate_confirm_pin_sent).days > 0:
            request.user.send_phone_number_confirmation_sms()
            request.user.send_phone_number_candidate_confirmation_sms()
            return Response({})
        return Response({'non_field_errors': ['too_frequent_sms_request']}, status=status.HTTP_400_BAD_REQUEST)


class RequestEmailConfirmPin(generics.GenericAPIView):
    serializer_class = ShortCustomUserSerializer

    def post(self, request):
        request.user.send_email_confirmation_email()
        request.user.send_email_candidate_confirmation_email()
        return Response({})


class GetUserRoles(views.APIView):
    def get(self, request):
        query_set = Group.objects.filter(user=request.user)
        return Response(query_set.all().values())


class UserByPhoneOrEmail(generics.GenericAPIView):
    permission_classes = (IsAuthenticated,)

    def get(self, request):
        phone_or_email = request.GET.get('value', None)
        user = CustomUser.objects.filter(Q(email=phone_or_email) | Q(phone_number=phone_or_email)).first()
        if user:
            return Response(ShortCustomUserSerializer(user).data)
        else:
            return Response({'non_field_errors': ['user_not_found']}, status=status.HTTP_404_NOT_FOUND)



import random
import uuid

import pyotp as pyotp
from django.contrib.auth.base_user import AbstractBaseUser, BaseUserManager
from django.contrib.auth.models import PermissionsMixin
from django.core.mail import send_mail
from django.db import models
from django.utils import timezone, translation
from django.utils.timezone import now
from django.utils.translation import gettext as _

from moses.conf import settings as moses_settings
from django.conf import settings as django_settings


class CustomUserManager(BaseUserManager):
    use_in_migrations = True

    def _create_user(self, phone_number, password, **extra_fields):
        user = self.model(phone_number=phone_number, **extra_fields)
        user.set_password(password)
        user.save(using=self._db)
        return user

    def create_user(self, phone_number=None, password=None, **extra_fields):
        extra_fields.setdefault('is_staff', False)
        extra_fields.setdefault('is_superuser', False)
        extra_fields.setdefault('is_active', True)
        user = self._create_user(phone_number, password, **extra_fields)
        user.send_phone_number_confirmation_sms(generate_new=True)
        user.send_email_confirmation_email(generate_new=True)
        return user

    def create_superuser(self, phone_number, password, **extra_fields):
        extra_fields.setdefault('is_staff', True)
        extra_fields.setdefault('is_superuser', True)

        if extra_fields.get('is_staff') is not True:
            raise ValueError('Superuser must have is_staff=True.')
        if extra_fields.get('is_superuser') is not True:
            raise ValueError('Superuser must have is_superuser=True.')

        return self._create_user(phone_number, password, **extra_fields)


class CustomUser(AbstractBaseUser, PermissionsMixin):
    class Meta:
        verbose_name = _("User")
        verbose_name_plural = _("Users")

    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)

    email = models.EmailField(unique=True, blank=False)
    email_candidate = models.EmailField(blank=True, verbose_name=_("Email candidate"))
    is_email_confirmed = models.BooleanField(default=False, verbose_name=_("Is email confirmed"))
    email_confirm_pin = models.PositiveIntegerField(default=0, verbose_name=_("Email confirm PIN"))
    email_candidate_confirm_pin = models.PositiveIntegerField(default=0, verbose_name=_("Email candidate confirm PIN"))
    email_confirm_attempts = models.PositiveSmallIntegerField(default=0, verbose_name=_("Email confirm attempts"))

    phone_number = models.CharField(max_length=200, unique=True, verbose_name=_("Phone number"))
    phone_number_candidate = models.CharField(max_length=200, blank=True,
                                              verbose_name=_("Phone number candidate"))
    is_phone_number_confirmed = models.BooleanField(default=False, verbose_name=_("Is phone number confirmed"))
    phone_number_confirm_pin = models.PositiveIntegerField(default=0, verbose_name=_("Phone number confirm PIN"))
    phone_number_candidate_confirm_pin = models.PositiveIntegerField(default=0, verbose_name=_(
        "Phone number candidate confirm PIN"))
    phone_number_confirm_attempts = models.PositiveSmallIntegerField(default=0, verbose_name=_("Phone number confirm attempts"))
    last_password_reset_sms_sent_at = models.DateTimeField(blank=True, null=True,
                                                           verbose_name=_("Last password reset sms sent at"))

    first_name = models.CharField(max_length=200, verbose_name=_("First name"), blank=True)
    last_name = models.CharField(max_length=200, verbose_name=_("Last name"), blank=True)

    is_active = models.BooleanField(default=True, verbose_name=_("Is active"))
    is_staff = models.BooleanField(default=False, verbose_name=_("Is staff"))

    preferred_language = models.CharField(choices=moses_settings.LANGUAGE_CHOICES,
                                          default=moses_settings.DEFAULT_LANGUAGE,
                                          max_length=10,
                                          verbose_name=_("Preferred language"))
    created_at = models.DateTimeField(default=timezone.now, blank=True, null=True, verbose_name=_("Created at"))

    mfa_secret_key = models.CharField(blank=True, default='', max_length=160)
    last_phone_number_confirm_pin_sent = models.DateTimeField(default=now)
    last_phone_number_candidate_confirm_pin_sent = models.DateTimeField(default=now)
    userpic = models.ImageField(upload_to='images/userpics/', blank=True, null=True)

    USERNAME_FIELD = 'phone_number'

    objects = CustomUserManager()

    def check_mfa_otp(self, otp):
        if self.is_superuser and not django_settings.DEBUG and not self.mfa_secret_key:
            return False
        elif not self.mfa_secret_key:
            return True
        totp = pyotp.totp.TOTP(self.mfa_secret_key.encode('utf-8'))
        return totp.verify(otp)

    def send_phone_number_confirmation_sms(self, generate_new=False):
        if generate_new or self.phone_number_confirm_pin == 0:
            self.phone_number_confirm_pin = random.randint(0, 999999)
        self.last_phone_number_confirm_pin_sent = timezone.now()
        self.save()
        with translation.override(self.preferred_language):
            moses_settings.SEND_SMS_HANDLER(self.phone_number,
                                            _("Phone number confirmation PIN: ") + str(
                                                self.phone_number_confirm_pin).zfill(6))

    def send_phone_number_candidate_confirmation_sms(self, generate_new=False):
        if not self.phone_number_candidate:
            return
        if generate_new or self.phone_number_candidate_confirm_pin == 0:
            self.phone_number_candidate_confirm_pin = random.randint(0, 999999)
        self.last_phone_number_candidate_confirm_pin_sent = timezone.now()
        self.save()
        with translation.override(self.preferred_language):
            moses_settings.SEND_SMS_HANDLER(self.phone_number_candidate,
                                            _("Phone number confirmation PIN: ") + str(
                                                self.phone_number_candidate_confirm_pin).zfill(
                                                6))

    def try_to_confirm_email(self, main_pin_str: str, candidate_pin_str: str):
        if self.email_confirm_attempts > 4:
            return False
        try:
            received_pin, received_candidate_pin = int(main_pin_str), int(candidate_pin_str or '0')
            if received_pin == self.email_confirm_pin and (
                    not self.email_candidate or self.email_candidate_confirm_pin == received_candidate_pin):
                self.email_confirm_pin = self.email_candidate_confirm_pin = 0
                self.is_email_confirmed = True
                self.email_confirm_attempts = 0
                if self.email_candidate:
                    self.email = self.email_candidate
                    self.email_candidate = ''
                self.save()
                return True
            else:
                self.email_confirm_attempts += 1
                self.save()
                return False
        except ValueError:
            self.email_confirm_attempts += 1
            self.save()
            return False

    def try_to_confirm_phone_number(self, main_pin_str: str, candidate_pin_str: str):
        try:
            received_pin, received_candidate_pin = int(main_pin_str), int(candidate_pin_str or '0')
            if received_pin == self.phone_number_confirm_pin and \
                    (not self.phone_number_candidate or
                     self.phone_number_candidate_confirm_pin == received_candidate_pin):
                self.phone_number_confirm_pin = 0
                self.is_phone_number_confirmed = True
                self.phone_number_confirm_attempts = 0
                if self.phone_number_candidate:
                    self.phone_number = self.phone_number_candidate
                    self.phone_number_candidate = ''
                self.save()
                return True
            else:
                return False
        except ValueError:
            self.phone_number_confirm_attempts += 1
            self.save()
            return False

    def send_email_confirmation_email(self, generate_new=False):
        if generate_new or self.email_confirm_pin == 0:
            self.email_confirm_pin = random.randint(0, 999999)
            self.save()
        with translation.override(self.preferred_language):
            send_mail(_("Email confirmation PIN"), _("Your email confirmation PIN is: ") + str(self.email_confirm_pin),
                      'noreply@' + moses_settings.DOMAIN, [self.email])

    def send_payeer_wallet_changed_email(self):
        with translation.override(self.preferred_language):
            send_mail(_("Payeer wallet has been updated"), _("Your payeer wallet has been updated "),
                      'noreply@' + moses_settings.DOMAIN, [self.email])

    def send_email_candidate_confirmation_email(self, generate_new=False):
        if not self.email_candidate:
            return
        if generate_new or self.email_candidate_confirm_pin == 0:
            self.email_candidate_confirm_pin = random.randint(0, 999999)
            self.save()
        with translation.override(self.preferred_language):
            send_mail(_("Email confirmation PIN"),
                      _("Your email confirmation PIN is: ") + str(self.email_candidate_confirm_pin),
                      'noreply@' + django_settings.DOMAIN, [self.email_candidate])

    def __str__(self):
        return f'{self.first_name} {self.last_name}'



from django.urls import path, include

from . import views as moses_views
from djoser import views as djoser_views
from rest_framework_simplejwt.views import (
    TokenRefreshView
)

app_name = 'moses'

urlpatterns = [
    path('token/obtain/', moses_views.TokenObtainPairView.as_view(), name='token_obtain_pair'),
    path('token/validate_auth/', moses_views.ValidateAuthView.as_view(), name='token_verify_pair'),
    path('token/refresh/', TokenRefreshView.as_view(), name='token_refresh'),

    path('mfa/', moses_views.MFAView.as_view(), name='mfa'),

    path('confirm_phone_number/', moses_views.ConfirmPhoneNumber.as_view(), name='confirm_phone_number'),
    path('confirm_email/', moses_views.ConfirmEmail.as_view(), name='confirm_phone_number'),

    path('request_phone_number_pin/', moses_views.RequestPhoneNumberConfirmPin.as_view(),
         name='request_phone_number_confirm_pin'),
    path('request_email_pin/', moses_views.RequestEmailConfirmPin.as_view(), name='request_email_confirm_pin'),

    path('password/', moses_views.SetPasswordView.as_view(), name='set_password'),
    path('password/reset/', moses_views.ResetPassword.as_view(), name='reset-password'),
    path('password/reset/confirm/', djoser_views.UserViewSet.as_view({'post': 'reset_password_confirm'}), name='reset-password-confirm'),
    path('is_email_available/', moses_views.CheckEmailAvailability.as_view(),
         name='check_email_availability'),
    path('is_mfa_enabled_for_phone_number/', moses_views.CheckIsMFAEnabled.as_view(),
         name='check_is_mfa_enabled'),
    path('token/', moses_views.TokenObtainPairView.as_view(), name='token_obtain_pair'),
    path('token/refresh/', TokenRefreshView.as_view(), name='token_refresh'),

    path('roles/', moses_views.GetUserRoles.as_view(), name='user_roles'),

    path('user_by_phone_or_email/', moses_views.UserByPhoneOrEmail.as_view(), name='get_user_by_phone_or_email'),

    path('', include('djoser.urls'))
]



# Generated by Django 4.2.1 on 2023-06-03 04:08

from django.db import migrations, models
import django.utils.timezone
import moses.models
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("auth", "0012_alter_user_first_name_max_length"),
    ]

    operations = [
        migrations.CreateModel(
            name="CustomUser",
            fields=[
                ("password", models.CharField(max_length=128, verbose_name="password")),
                (
                    "last_login",
                    models.DateTimeField(
                        blank=True, null=True, verbose_name="last login"
                    ),
                ),
                (
                    "is_superuser",
                    models.BooleanField(
                        default=False,
                        help_text="Designates that this user has all permissions without explicitly assigning them.",
                        verbose_name="superuser status",
                    ),
                ),
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("email", models.EmailField(max_length=254, unique=True)),
                (
                    "email_candidate",
                    models.EmailField(
                        blank=True, max_length=254, verbose_name="Email candidate"
                    ),
                ),
                (
                    "is_email_confirmed",
                    models.BooleanField(
                        default=False, verbose_name="Is email confirmed"
                    ),
                ),
                (
                    "email_confirm_pin",
                    models.PositiveIntegerField(
                        default=0, verbose_name="Email confirm PIN"
                    ),
                ),
                (
                    "email_candidate_confirm_pin",
                    models.PositiveIntegerField(
                        default=0, verbose_name="Email candidate confirm PIN"
                    ),
                ),
                (
                    "email_confirm_attempts",
                    models.PositiveSmallIntegerField(
                        default=0, verbose_name="Email confirm attempts"
                    ),
                ),
                (
                    "phone_number",
                    models.CharField(
                        max_length=200, unique=True, verbose_name="Phone number"
                    ),
                ),
                (
                    "phone_number_candidate",
                    models.CharField(
                        blank=True,
                        max_length=200,
                        verbose_name="Phone number candidate",
                    ),
                ),
                (
                    "is_phone_number_confirmed",
                    models.BooleanField(
                        default=False, verbose_name="Is phone number confirmed"
                    ),
                ),
                (
                    "phone_number_confirm_pin",
                    models.PositiveIntegerField(
                        default=0, verbose_name="Phone number confirm PIN"
                    ),
                ),
                (
                    "phone_number_candidate_confirm_pin",
                    models.PositiveIntegerField(
                        default=0, verbose_name="Phone number candidate confirm PIN"
                    ),
                ),
                (
                    "phone_number_confirm_attempts",
                    models.PositiveSmallIntegerField(
                        default=0, verbose_name="Phone number confirm attempts"
                    ),
                ),
                (
                    "last_password_reset_sms_sent_at",
                    models.DateTimeField(
                        blank=True,
                        null=True,
                        verbose_name="Last password reset sms sent at",
                    ),
                ),
                (
                    "first_name",
                    models.CharField(
                        blank=True, max_length=200, verbose_name="First name"
                    ),
                ),
                (
                    "last_name",
                    models.CharField(
                        blank=True, max_length=200, verbose_name="Last name"
                    ),
                ),
                (
                    "is_active",
                    models.BooleanField(default=True, verbose_name="Is active"),
                ),
                (
                    "is_staff",
                    models.BooleanField(default=False, verbose_name="Is staff"),
                ),
                (
                    "preferred_language",
                    models.CharField(
                        choices=[("en", "English")],
                        default="en",
                        max_length=10,
                        verbose_name="Preferred language",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        blank=True,
                        default=django.utils.timezone.now,
                        null=True,
                        verbose_name="Created at",
                    ),
                ),
                (
                    "mfa_secret_key",
                    models.CharField(blank=True, default="", max_length=160),
                ),
                (
                    "last_phone_number_confirm_pin_sent",
                    models.DateTimeField(default=django.utils.timezone.now),
                ),
                (
                    "last_phone_number_candidate_confirm_pin_sent",
                    models.DateTimeField(default=django.utils.timezone.now),
                ),
                (
                    "userpic",
                    models.ImageField(
                        blank=True, null=True, upload_to="images/userpics/"
                    ),
                ),
                (
                    "groups",
                    models.ManyToManyField(
                        blank=True,
                        help_text="The groups this user belongs to. A user will get all permissions granted to each of their groups.",
                        related_name="user_set",
                        related_query_name="user",
                        to="auth.group",
                        verbose_name="groups",
                    ),
                ),
                (
                    "user_permissions",
                    models.ManyToManyField(
                        blank=True,
                        help_text="Specific permissions for this user.",
                        related_name="user_set",
                        related_query_name="user",
                        to="auth.permission",
                        verbose_name="user permissions",
                    ),
                ),
            ],
            options={
                "verbose_name": "User",
                "verbose_name_plural": "Users",
            },
            managers=[
                ("objects", moses.models.CustomUserManager()),
            ],
        ),
    ]
